"""
Author: Ryan Cocking (bsrctb@leeds.ac.uk)

FFEA web tutorial: https://ffea.readthedocs.io/en/stable/pdbtoemmaptut.html

Provided files: 4hel.pdb, EMD-5043.map

This script follows the web tutorial step-by-step, allowing you to quickly
generate an 8 angstrom volumetric mesh and FFEA script file from an electron
density map (EDM) of the GroEL protein

A simulation of GroEL will run briefly for 1 nanosecond, after which the
trajectory can be viewed using PyMol 1.8.x with the FFEA plugin.
"""
import os

# Start from the GroEL PDB file: https://www.rcsb.org/structure/4HEL
print("1 Obtaining an EDM (electron density map")
os.system("ffeatools pdbtoemmap 4hel.pdb 4hel_pdbtoem.map 50 50 50 25")
print("================================================================")

# At this point, the tutorial switches to use a lower-resolution (i.e. smoother) EDM map: https://www.ebi.ac.uk/pdbe/entry/emdb/EMD-5043
print("2 EDM to Surface Profile")
os.system("ffeatools emmaptosurf -map EMD-5043.map -out EMD-5043_processed.map -format map -level 1.15 -cull_floaters 10000 -fill_cavities 10000")
os.system("ffeatools emmaptosurf -map EMD-5043_processed.map -out EMD-5043.obj -format obj -level 1.15")
print("================================================================")

print("3 Coarsening a Surface Profile")
os.system("ffeatools surftocgsurf EMD-5043.obj EMD-5043_8ang.stl 8 y y")
print("================================================================")

print("4 Surface Profile to Volumetric Mesh")
os.system("tetgen -Y EMD-5043_8ang.stl")
print("================================================================")

print("5 Volumetric Mesh to FFEA (Tetgen)")
os.system("ffeatools voltoffea --mesh EMD-5043_8ang.1.ele EMD-5043_8ang.1.node EMD-5043_8ang.1.face --density 1.5e3 --shear-visc 1e-3  --bulk-visc 1e-3 --shear-mod 5.5e8 --bulk-mod 2.2e9 --dielec 1.0 --make-script")
# In-place text editing of FFEA script
os.system("sed -i '3i\        <dt = 1e-15>' EMD-5043_8ang.ffea")
os.system("sed -i '4i\        <num_steps = 1000>' EMD-5043_8ang.ffea")
os.system("sed -i '5i\        <check = 10>' EMD-5043_8ang.ffea")
print("Edited FFEA script to run simulations for 1 nanosecond")
print("================================================================")

print("6 Running a simulation")
os.system("export OMP_NUM_THREADS=2")
os.system("ffea EMD-5043_8ang.ffea")
